package org.usfirst.frc.team3501.robot.commands.auton;

import org.usfirst.frc.team3501.robot.Constants;
import org.usfirst.frc.team3501.robot.commands.driving.DriveDistance;
import org.usfirst.frc.team3501.robot.commands.driving.DriveForTime;
import org.usfirst.frc.team3501.robot.commands.driving.TurnForAngle;

import edu.wpi.first.wpilibj.command.CommandGroup;

/**
 * This command group will be used in autonomous. Based on what position the
 * robot is in, the robot will align with the goal. In the Software 2015-2016
 * Google folder is a picture explaining each of the cases.
 *
 * dependency on sensors: lidars, encoders, gyro
 *
 * dependency on subsystems: drivetrain
 *
 * dependency on other commands: TurnForAngle(), DriveForDistance()
 *
 * pre-condition: robot is flush against a defense at the specified position in
 * the opponent's courtyard
 *
 * post-condition: the robot is parallel to one of the three goals and the
 * shooter is facing that goal
 *
 */
public class AlignToScore extends CommandGroup {

  private final double DEFAULT_SPEED = 0.5;
  private final double maxTimeout = 5;

  // in inches
  // assuming that positive angle means turning right
  // and negative angle means turning left

  // constants for position 1: low bar

  public double horizontalDistToGoal;

  public AlignToScore(int position) {
    if (Constants.Auton.IS_USING_TIME) {
      if (position == 1) {
        addSequential(new DriveForTime(Constants.Auton.POS1_DIST1_TIME,
            Constants.Auton.POS1_DRIVE_MAXSPEED));
        addSequential(new TurnForAngle(Constants.Auton.POS1_TURN1,
            Constants.Auton.POS1_TURN_MAXSPEED));
        addSequential(new DriveDistance(Constants.Auton.POS1_DIST2_TIME,
            Constants.Auton.POS1_DRIVE_MAXSPEED));
        horizontalDistToGoal = 0;
      }
      else if (position == 2) {
        addSequential(new DriveForTime(Constants.Auton.POS2_DIST1_TIME,
            Constants.Auton.POS2_DRIVE_MAXSPEED));
        addSequential(new TurnForAngle(Constants.Auton.POS2_TURN1,
            Constants.Auton.POS2_TURN_MAXSPEED));
        addSequential(new DriveDistance(Constants.Auton.POS2_DIST2_TIME,
            Constants.Auton.POS2_DRIVE_MAXSPEED));
        horizontalDistToGoal = 0;
      }
      else if (position == 3) {
        addSequential(new DriveForTime(Constants.Auton.POS3_DIST1_TIME,
            Constants.Auton.POS3_DRIVE_MAXSPEED));
        addSequential(new TurnForAngle(Constants.Auton.POS3_TURN1,
            Constants.Auton.POS3_TURN_MAXSPEED));
        addSequential(new DriveDistance(Constants.Auton.POS3_DIST2_TIME,
            Constants.Auton.POS3_DRIVE_MAXSPEED));
        horizontalDistToGoal = 0;
      }
      else if (position == 4) {
        addSequential(new DriveForTime(Constants.Auton.POS4_DIST1_TIME,
            Constants.Auton.POS4_DRIVE_MAXSPEED));
        addSequential(new TurnForAngle(Constants.Auton.POS4_TURN1,
            Constants.Auton.POS4_TURN_MAXSPEED));
        addSequential(new DriveDistance(Constants.Auton.POS4_DIST2_TIME,
            Constants.Auton.POS4_DRIVE_MAXSPEED));
        horizontalDistToGoal = 0;
      }
      else if (position == 5) {
        addSequential(new DriveForTime(Constants.Auton.POS5_DIST1_TIME,
            Constants.Auton.POS5_DRIVE_MAXSPEED));
        addSequential(new TurnForAngle(Constants.Auton.POS5_TURN1,
            Constants.Auton.POS5_TURN_MAXSPEED));
        addSequential(new DriveDistance(Constants.Auton.POS5_DIST2_TIME,
            Constants.Auton.POS5_DRIVE_MAXSPEED));
        horizontalDistToGoal = 0;
      }
    }
    else {
      // position 1 is always the low bar
      if (position == 1) {
        addSequential(new DriveDistance(Constants.Auton.POS1_DIST1,
            Constants.Auton.DRIVE_MAX_TIMEOUT));
        addSequential(new TurnForAngle(Constants.Auton.POS1_TURN1,
            Constants.Auton.TURN_MAX_TIMEOUT));
        addSequential(new DriveDistance(Constants.Auton.POS1_DIST2,
            Constants.Auton.DRIVE_MAX_TIMEOUT));
      } else if (position == 2) {

        addSequential(new DriveDistance(Constants.Auton.POS2_DIST1,
            Constants.Auton.DRIVE_MAX_TIMEOUT));
        addSequential(new TurnForAngle(Constants.Auton.POS2_TURN1,
            Constants.Auton.TURN_MAX_TIMEOUT));
        addSequential(new DriveDistance(Constants.Auton.POS2_DIST2,
            Constants.Auton.DRIVE_MAX_TIMEOUT));

      } else if (position == 3) {

        addSequential(new DriveDistance(Constants.Auton.POS3_DIST1,
            Constants.Auton.DRIVE_MAX_TIMEOUT));
        addSequential(new TurnForAngle(Constants.Auton.POS3_TURN1, maxTimeout));
        addSequential(new DriveDistance(Constants.Auton.POS3_DIST2,
            Constants.Auton.TURN_MAX_TIMEOUT));
        addSequential(new TurnForAngle(Constants.Auton.POS3_TURN2, maxTimeout));
        addSequential(new DriveDistance(Constants.Auton.POS3_DIST3,
            Constants.Auton.DRIVE_MAX_TIMEOUT));

      } else if (position == 4) {

        addSequential(new DriveDistance(Constants.Auton.POS4_DIST1,
            Constants.Auton.DRIVE_MAX_TIMEOUT));
        addSequential(new TurnForAngle(Constants.Auton.POS4_TURN1, maxTimeout));
        addSequential(new DriveDistance(Constants.Auton.POS4_DIST2,
            Constants.Auton.TURN_MAX_TIMEOUT));
        addSequential(new TurnForAngle(Constants.Auton.POS4_TURN2, maxTimeout));
        addSequential(new DriveDistance(Constants.Auton.POS4_DIST3,
            Constants.Auton.DRIVE_MAX_TIMEOUT));

      } else if (position == 5) {

        addSequential(new DriveDistance(Constants.Auton.POS5_DIST1,
            Constants.Auton.DRIVE_MAX_TIMEOUT));
        addSequential(new TurnForAngle(Constants.Auton.POS5_TURN1, maxTimeout));
        addSequential(new DriveDistance(Constants.Auton.POS5_DIST2,
            Constants.Auton.TURN_MAX_TIMEOUT));
        addSequential(new TurnForAngle(Constants.Auton.POS5_TURN2, maxTimeout));
        addSequential(new DriveDistance(Constants.Auton.POS5_DIST3,
            Constants.Auton.DRIVE_MAX_TIMEOUT));
      }
    }
  }
  // following commented out method is calculations for path of robot in auton
  // after passing through defense using two lidars
  /*
   * public static double lidarCalculateAngleToTurn(int position,
   * double horizontalDistToGoal) {
   * double leftDist = Robot.driveTrain.getLeftLidarDistance();
   * double rightDist = Robot.driveTrain.getRightLidarDistance();
   * 
   * double errorAngle = Math.atan(Math.abs(leftDist - rightDist) / 2);
   * double distToTower;
   * // TODO: figure out if we do want to shoot into the side goal if we are
   * // in position 1 or 2, or if we want to change that
   * if (position == 1 || position == 2) {
   * distToTower = Math
   * .cos(CENTER_OF_MASS_TO_ROBOT_FRONT + (leftDist - rightDist) / 2)
   * - DIST_CASTLE_WALL_TO_SIDE_GOAL;
   * }
   * 
   * // TODO: figure out if we do want to shoot into the font goal if we are
   * // in position 3, 4, 5, or if we want to change that
   * else {
   * distToTower = Math
   * .cos(CENTER_OF_MASS_TO_ROBOT_FRONT + (leftDist - rightDist) / 2)
   * - DIST_CASTLE_WALL_TO_SIDE_GOAL;
   * }
   * 
   * double angleToTurn = Math.atan(distToTower / horizontalDistToGoal);
   * 
   * return angleToTurn;
   * }
   */
}
